Package: sp
Version: 2.2-0
Title: Classes and Methods for Spatial Data
Authors@R: c(person("Edzer", "Pebesma", role = c("aut", "cre"),
				email = "edzer.pebesma@uni-muenster.de"),
			person("Roger", "Bivand", role = "aut",
            	email = "Roger.Bivand@nhh.no"),
			person("Barry", "Rowlingson", role = "ctb"),
			person("Virgilio", "Gomez-Rubio", role = "ctb"),
			person("Robert", "Hijmans", role = "ctb"),
			person("Michael", "Sumner", role = "ctb"),
			person("Don", "MacQueen", role = "ctb"),
			person("Jim", "Lemon", role = "ctb"),
                        person("Finn", "Lindgren", role = "ctb"),
			person("Josh", "O'Brien", role = "ctb"),
			person("Joseph", "O'Rourke", role = "ctb"),
                        person("Patrick", "Hausmann", role = "ctb"))
Depends: R (>= 3.5.0), methods
Imports: utils, stats, graphics, grDevices, lattice, grid
Suggests: RColorBrewer, gstat, deldir, knitr, maps, mapview, rmarkdown,
        sf, terra, raster
Description: Classes and methods for spatial
  data; the classes document where the spatial location information
  resides, for 2D or 3D data. Utility functions are provided, e.g. for
  plotting data as maps, spatial selection, as well as methods for
  retrieving coordinates, for subsetting, print, summary, etc. From this
  version, 'rgdal', 'maptools', and 'rgeos' are no longer used at all,
  see <https://r-spatial.org/r/2023/05/15/evolution4.html> for details.
License: GPL (>= 2)
URL: https://github.com/edzer/sp/ https://edzer.github.io/sp/
BugReports: https://github.com/edzer/sp/issues
Collate: bpy.colors.R AAA.R Class-CRS.R CRS-methods.R Class-Spatial.R
        Spatial-methods.R projected.R Class-SpatialPoints.R
        SpatialPoints-methods.R Class-SpatialPointsDataFrame.R
        SpatialPointsDataFrame-methods.R Class-SpatialMultiPoints.R
        SpatialMultiPoints-methods.R
        Class-SpatialMultiPointsDataFrame.R
        SpatialMultiPointsDataFrame-methods.R Class-GridTopology.R
        Class-SpatialGrid.R Class-SpatialGridDataFrame.R
        Class-SpatialLines.R SpatialLines-methods.R
        Class-SpatialLinesDataFrame.R SpatialLinesDataFrame-methods.R
        Class-SpatialPolygons.R Class-SpatialPolygonsDataFrame.R
        SpatialPolygons-methods.R SpatialPolygonsDataFrame-methods.R
        GridTopology-methods.R SpatialGrid-methods.R
        SpatialGridDataFrame-methods.R SpatialPolygons-internals.R
        point.in.polygon.R SpatialPolygons-displayMethods.R zerodist.R
        image.R stack.R bubble.R mapasp.R select.spatial.R gridded.R
        asciigrid.R spplot.R over.R spsample.R recenter.R dms.R
        gridlines.R spdists.R rbind.R flipSGDF.R chfids.R loadmeuse.R
        compassRose.R surfaceArea.R spOptions.R subset.R disaggregate.R
        sp_spat1.R merge.R aggregate.R elide.R sp2Mondrian.R
VignetteBuilder: knitr
NeedsCompilation: yes
Packaged: 2025-01-30 11:06:55 UTC; edzer
Author: Edzer Pebesma [aut, cre],
  Roger Bivand [aut],
  Barry Rowlingson [ctb],
  Virgilio Gomez-Rubio [ctb],
  Robert Hijmans [ctb],
  Michael Sumner [ctb],
  Don MacQueen [ctb],
  Jim Lemon [ctb],
  Finn Lindgren [ctb],
  Josh O'Brien [ctb],
  Joseph O'Rourke [ctb],
  Patrick Hausmann [ctb]
Maintainer: Edzer Pebesma <edzer.pebesma@uni-muenster.de>
Repository: CRAN
Date/Publication: 2025-02-01 09:40:02 UTC
Built: R 4.4.0; amd64-portbld-midnightbsd4.0.2; 2026-02-17 17:59:00 UTC; unix
