Package: sp
Version: 1.4-7
Title: Classes and Methods for Spatial Data
Authors@R: c(person("Edzer", "Pebesma", role = c("aut", "cre"),
				email = "edzer.pebesma@uni-muenster.de"),
			person("Roger", "Bivand", role = "aut",
            	email = "Roger.Bivand@nhh.no"),
			person("Barry", "Rowlingson", role = "ctb"),
			person("Virgilio", "Gomez-Rubio", role = "ctb"),
			person("Robert", "Hijmans", role = "ctb"),
			person("Michael", "Sumner", role = "ctb"),
			person("Don", "MacQueen", role = "ctb"),
			person("Jim", "Lemon", role = "ctb"),
                        person("Finn", "Lindgren", role = "ctb"),
			person("Josh", "O'Brien", role = "ctb"),
			person("Joseph", "O'Rourke", role = "ctb"))
Depends: R (>= 3.0.0), methods
Imports: utils, stats, graphics, grDevices, lattice, grid
Suggests: RColorBrewer, rgdal (>= 1.2-3), rgeos (>= 0.3-13), gstat,
        maptools, deldir, knitr, rmarkdown
Description: Classes and methods for spatial
  data; the classes document where the spatial location information
  resides, for 2D or 3D data. Utility functions are provided, e.g. for
  plotting data as maps, spatial selection, as well as methods for
  retrieving coordinates, for subsetting, print, summary, etc.
License: GPL (>= 2)
URL: https://github.com/edzer/sp/ https://edzer.github.io/sp/
BugReports: https://github.com/edzer/sp/issues
Collate: bpy.colors.R AAA.R Class-CRS.R CRS-methods.R Class-Spatial.R
        Spatial-methods.R projected.R Class-SpatialPoints.R
        SpatialPoints-methods.R Class-SpatialPointsDataFrame.R
        SpatialPointsDataFrame-methods.R Class-SpatialMultiPoints.R
        SpatialMultiPoints-methods.R
        Class-SpatialMultiPointsDataFrame.R
        SpatialMultiPointsDataFrame-methods.R Class-GridTopology.R
        Class-SpatialGrid.R Class-SpatialGridDataFrame.R
        Class-SpatialLines.R SpatialLines-methods.R
        Class-SpatialLinesDataFrame.R SpatialLinesDataFrame-methods.R
        Class-SpatialPolygons.R Class-SpatialPolygonsDataFrame.R
        SpatialPolygons-methods.R SpatialPolygonsDataFrame-methods.R
        GridTopology-methods.R SpatialGrid-methods.R
        SpatialGridDataFrame-methods.R SpatialPolygons-internals.R
        point.in.polygon.R SpatialPolygons-displayMethods.R zerodist.R
        image.R stack.R bubble.R mapasp.R select.spatial.R gridded.R
        asciigrid.R spplot.R over.R spsample.R recenter.R dms.R
        gridlines.R spdists.R rbind.R flipSGDF.R chfids.R loadmeuse.R
        compassRose.R surfaceArea.R spOptions.R subset.R disaggregate.R
        sp_spat1.R merge.R aggregate.R
VignetteBuilder: knitr
NeedsCompilation: yes
Packaged: 2022-04-18 21:16:35 UTC; edzer
Author: Edzer Pebesma [aut, cre],
  Roger Bivand [aut],
  Barry Rowlingson [ctb],
  Virgilio Gomez-Rubio [ctb],
  Robert Hijmans [ctb],
  Michael Sumner [ctb],
  Don MacQueen [ctb],
  Jim Lemon [ctb],
  Finn Lindgren [ctb],
  Josh O'Brien [ctb],
  Joseph O'Rourke [ctb]
Maintainer: Edzer Pebesma <edzer.pebesma@uni-muenster.de>
Repository: CRAN
Date/Publication: 2022-04-20 10:00:02 UTC
